@@ -16,13 +16,18 @@
 #define INCLUDED_SRC_BUILDTOOL_BUILD_ENGINE_EXPRESSION_CONFIGURATION_HPP
 
 #include <algorithm>
+#include <compare>
 #include <cstddef>
+#include <functional>
+#include <optional>
 #include <sstream>
 #include <string>
 #include <utility>  // std::move
+#include <vector>
 
-#include "gsl/gsl"
+#include "nlohmann/json.hpp"
 #include "src/buildtool/build_engine/expression/expression.hpp"
+#include "src/buildtool/build_engine/expression/expression_ptr.hpp"
 #include "src/utils/cpp/concepts.hpp"
 #include "src/utils/cpp/gsl.hpp"
 
@@ -88,6 +93,10 @@ class Configuration {
         return expr_ == other.expr_;
     }
 
+    [[nodiscard]] auto operator<(const Configuration& other) const -> bool {
+        return expr_->ToHash() < other.expr_->ToHash();
+    }
+
     [[nodiscard]] auto hash() const noexcept -> std::size_t {
         return std::hash<ExpressionPtr>{}(expr_);
     }
