@@ -52,6 +52,7 @@
 #include "src/buildtool/common/artifact_digest_factory.hpp"
 #include "src/buildtool/common/repository_config.hpp"
 #include "src/buildtool/common/tree.hpp"
+#include "src/buildtool/common/tree_overlay.hpp"
 #include "src/buildtool/file_system/object_type.hpp"
 #include "src/buildtool/storage/storage.hpp"
 #include "src/utils/cpp/path.hpp"
@@ -85,7 +86,6 @@ auto const kTreeRuleFields = std::unordered_set<std::string>{"arguments_config",
                                                              "name",
                                                              "tainted",
                                                              "type"};
-
 auto const kInstallRuleFields =
     std::unordered_set<std::string>{"arguments_config",
                                     "deps",
@@ -301,6 +301,7 @@ void BlobGenRuleWithDeps(
         std::vector<ActionDescription::Ptr>{},
         std::vector<std::string>{data_val->String()},
         std::vector<Tree::Ptr>{},
+        std::vector<TreeOverlay::Ptr>{},
         std::move(vars_set),
         std::move(tainted),
         std::move(implied_export),
@@ -443,7 +444,8 @@ void TreeRuleWithDeps(
     const BuildMaps::Base::FieldReader::Ptr& desc,
     const BuildMaps::Target::TargetMap::SetterPtr& setter,
     const BuildMaps::Target::TargetMap::LoggerPtr& logger,
-    const gsl::not_null<BuildMaps::Target::ResultTargetMap*>& result_map) {
+    const gsl::not_null<BuildMaps::Target::ResultTargetMap*>& result_map,
+    std::optional<bool> disjoint_overlay) {
     auto param_vars = desc->ReadStringList("arguments_config");
     if (not param_vars) {
         return;
@@ -495,45 +497,75 @@ void TreeRuleWithDeps(
         {},
         {}};
 
-    // Compute the stage
-    auto stage = ExpressionPtr{Expression::map_t{}};
-    for (auto const& dep : dependency_values) {
-        auto to_stage = ExpressionPtr{
-            Expression::map_t{(*dep)->RunFiles(), (*dep)->Artifacts()}};
-        auto dup = stage->Map().FindConflictingDuplicate(to_stage->Map());
-        if (dup) {
-            std::unordered_map<BuildMaps::Base::EntityName, AnalysedTargetPtr>
-                deps_by_target;
-            deps_by_target.reserve(dependency_keys.size());
-            for (std::size_t i = 0; i < dependency_keys.size(); ++i) {
-                deps_by_target.emplace(dependency_keys[i].target,
-                                       *dependency_values[i]);
+    // Compute the resulting stage
+    auto result_stage = Expression::map_t::underlying_map_t{};
+    std::vector<Tree::Ptr> trees{};
+    std::vector<TreeOverlay::Ptr> tree_overlays{};
+
+    if (disjoint_overlay) {
+        TreeOverlay::to_overlay_t dep_trees{};
+        for (auto const& dep : dependency_values) {
+            std::unordered_map<std::string, ArtifactDescription> tree_content;
+            for (auto const& [input_path, artifact] :
+                 (*dep)->Artifacts()->Map()) {
+                auto norm_path =
+                    ToNormalPath(std::filesystem::path{input_path});
+                tree_content.emplace(std::move(norm_path),
+                                     artifact->Artifact());
             }
-            ReportStagingConflict(
-                dup->get(), stage, to_stage, deps_by_target, logger);
+            auto tree = std::make_shared<Tree>(std::move(tree_content));
+            auto tree_id = tree->Id();
+            trees.emplace_back(std::move(tree));
+            dep_trees.emplace_back(ArtifactDescription::CreateTree(tree_id));
+        }
+        auto overlay_tree = std::make_shared<TreeOverlay>(std::move(dep_trees),
+                                                          *disjoint_overlay);
+        auto overlay_tree_id = overlay_tree->Id();
+        tree_overlays.emplace_back(std::move(overlay_tree));
+        result_stage.emplace(
+            name, ArtifactDescription::CreateTreeOverlay(overlay_tree_id));
+    }
+    else {
+
+        auto stage = ExpressionPtr{Expression::map_t{}};
+        for (auto const& dep : dependency_values) {
+            auto to_stage = ExpressionPtr{
+                Expression::map_t{(*dep)->RunFiles(), (*dep)->Artifacts()}};
+            auto dup = stage->Map().FindConflictingDuplicate(to_stage->Map());
+            if (dup) {
+                std::unordered_map<BuildMaps::Base::EntityName,
+                                   AnalysedTargetPtr>
+                    deps_by_target;
+                deps_by_target.reserve(dependency_keys.size());
+                for (std::size_t i = 0; i < dependency_keys.size(); ++i) {
+                    deps_by_target.emplace(dependency_keys[i].target,
+                                           *dependency_values[i]);
+                }
+                ReportStagingConflict(
+                    dup->get(), stage, to_stage, deps_by_target, logger);
+                return;
+            }
+            stage = ExpressionPtr{Expression::map_t{stage, to_stage}};
+        }
+
+        // Result is the associated tree, located at name
+        auto conflict = BuildMaps::Target::Utils::tree_conflict(stage);
+        if (conflict) {
+            (*logger)(fmt::format("TREE conflict on subtree {}", *conflict),
+                      true);
             return;
         }
-        stage = ExpressionPtr{Expression::map_t{stage, to_stage}};
+        std::unordered_map<std::string, ArtifactDescription> tree_content;
+        tree_content.reserve(stage->Map().size());
+        for (auto const& [input_path, artifact] : stage->Map()) {
+            auto norm_path = ToNormalPath(std::filesystem::path{input_path});
+            tree_content.emplace(std::move(norm_path), artifact->Artifact());
+        }
+        auto tree = std::make_shared<Tree>(std::move(tree_content));
+        auto tree_id = tree->Id();
+        trees.emplace_back(std::move(tree));
+        result_stage.emplace(name, ArtifactDescription::CreateTree(tree_id));
     }
-
-    // Result is the associated tree, located at name
-    auto conflict = BuildMaps::Target::Utils::tree_conflict(stage);
-    if (conflict) {
-        (*logger)(fmt::format("TREE conflict on subtree {}", *conflict), true);
-        return;
-    }
-    std::unordered_map<std::string, ArtifactDescription> tree_content;
-    tree_content.reserve(stage->Map().size());
-    for (auto const& [input_path, artifact] : stage->Map()) {
-        auto norm_path = ToNormalPath(std::filesystem::path{input_path});
-        tree_content.emplace(std::move(norm_path), artifact->Artifact());
-    }
-    auto tree = std::make_shared<Tree>(std::move(tree_content));
-    auto tree_id = tree->Id();
-    std::vector<Tree::Ptr> trees{};
-    trees.emplace_back(std::move(tree));
-    auto result_stage = Expression::map_t::underlying_map_t{};
-    result_stage.emplace(name, ArtifactDescription::CreateTree(tree_id));
     auto result = ExpressionPtr{Expression::map_t{result_stage}};
 
     auto analysis_result = std::make_shared<AnalysedTarget const>(
@@ -543,6 +575,7 @@ void TreeRuleWithDeps(
         std::vector<ActionDescription::Ptr>{},
         std::vector<std::string>{},
         std::move(trees),
+        std::move(tree_overlays),
         std::move(vars_set),
         std::move(tainted),
         std::move(implied_export),
@@ -552,16 +585,25 @@ void TreeRuleWithDeps(
     (*setter)(std::move(analysis_result));
 }
 
-void TreeRule(
+void CommonTreeRule(
     const gsl::not_null<AnalyseContext*>& context,
     const nlohmann::json& desc_json,
     const BuildMaps::Target::ConfiguredTarget& key,
     const BuildMaps::Target::TargetMap::SubCallerPtr& subcaller,
     const BuildMaps::Target::TargetMap::SetterPtr& setter,
     const BuildMaps::Target::TargetMap::LoggerPtr& logger,
-    const gsl::not_null<BuildMaps::Target::ResultTargetMap*>& result_map) {
+    const gsl::not_null<BuildMaps::Target::ResultTargetMap*>& result_map,
+    std::optional<bool> disjoint_overlay) {
+    std::string rule_name;
+    if (disjoint_overlay) {
+        rule_name =
+            *disjoint_overlay ? "disjoint_tree_overlay" : "tree overlay";
+    }
+    else {
+        rule_name = "tree";
+    }
     auto desc = BuildMaps::Base::FieldReader::CreatePtr(
-        desc_json, key.target, "tree target", logger);
+        desc_json, key.target, fmt::format("{} target", rule_name), logger);
     desc->ExpectFields(kTreeRuleFields);
     auto param_vars = desc->ReadStringList("arguments_config");
     if (not param_vars) {
@@ -634,7 +676,8 @@ void TreeRule(
          setter,
          logger,
          key,
-         result_map](auto const& values) {
+         result_map,
+         disjoint_overlay](auto const& values) {
             TreeRuleWithDeps(values,
                              dependency_keys,
                              name,
@@ -642,11 +685,54 @@ void TreeRule(
                              desc,
                              setter,
                              logger,
-                             result_map);
+                             result_map,
+                             disjoint_overlay);
         },
         logger);
 }
 
+void TreeRule(
+    const gsl::not_null<AnalyseContext*>& context,
+    const nlohmann::json& desc_json,
+    const BuildMaps::Target::ConfiguredTarget& key,
+    const BuildMaps::Target::TargetMap::SubCallerPtr& subcaller,
+    const BuildMaps::Target::TargetMap::SetterPtr& setter,
+    const BuildMaps::Target::TargetMap::LoggerPtr& logger,
+    const gsl::not_null<BuildMaps::Target::ResultTargetMap*>& result_map) {
+    CommonTreeRule(context,
+                   desc_json,
+                   key,
+                   subcaller,
+                   setter,
+                   logger,
+                   result_map,
+                   std::nullopt);
+}
+
+void DisjointTreeOverlayRule(
+    const gsl::not_null<AnalyseContext*>& context,
+    const nlohmann::json& desc_json,
+    const BuildMaps::Target::ConfiguredTarget& key,
+    const BuildMaps::Target::TargetMap::SubCallerPtr& subcaller,
+    const BuildMaps::Target::TargetMap::SetterPtr& setter,
+    const BuildMaps::Target::TargetMap::LoggerPtr& logger,
+    const gsl::not_null<BuildMaps::Target::ResultTargetMap*>& result_map) {
+    CommonTreeRule(
+        context, desc_json, key, subcaller, setter, logger, result_map, true);
+}
+
+void TreeOverlayRule(
+    const gsl::not_null<AnalyseContext*>& context,
+    const nlohmann::json& desc_json,
+    const BuildMaps::Target::ConfiguredTarget& key,
+    const BuildMaps::Target::TargetMap::SubCallerPtr& subcaller,
+    const BuildMaps::Target::TargetMap::SetterPtr& setter,
+    const BuildMaps::Target::TargetMap::LoggerPtr& logger,
+    const gsl::not_null<BuildMaps::Target::ResultTargetMap*>& result_map) {
+    CommonTreeRule(
+        context, desc_json, key, subcaller, setter, logger, result_map, false);
+}
+
 void InstallRuleWithDeps(
     const std::vector<BuildMaps::Target::ConfiguredTarget>& dependency_keys,
     const std::vector<AnalysedTargetPtr const*>& dependency_values,
@@ -668,7 +754,7 @@ void InstallRuleWithDeps(
                                *dependency_values[i]);
     }
 
-    // Compute the effective dependecy on config variables
+    // Compute the effective dependency on config variables
     std::unordered_set<std::string> effective_vars;
     auto param_vars = desc->ReadStringList("arguments_config");
     effective_vars.insert(param_vars->begin(), param_vars->end());
@@ -771,7 +857,7 @@ void InstallRuleWithDeps(
         auto dir_path = std::filesystem::path{subdir.second};
         auto target = deps_by_target.at(subdir.first);
         // within a target, artifacts and runfiles may overlap, but artifacts
-        // take perference
+        // take preference
         for (auto const& [path, artifact] : target->Artifacts()->Map()) {
             subdir_stage.emplace(ToNormalPath(dir_path / path).string(),
                                  artifact);
@@ -802,6 +888,7 @@ void InstallRuleWithDeps(
         std::vector<ActionDescription::Ptr>{},
         std::vector<std::string>{},
         std::vector<Tree::Ptr>{},
+        std::vector<TreeOverlay::Ptr>{},
         std::move(effective_vars),
         std::move(tainted),
         std::move(implied_export),
@@ -1233,7 +1320,7 @@ void GenericRuleWithDeps(
         return;
     }
     if (not PathIsNonUpwards(cwd_value->String())) {
-        (*logger)(fmt::format("cwd has to evalute to a non-upwards relative "
+        (*logger)(fmt::format("cwd has to evaluate to a non-upwards relative "
                               "path, but found {}",
                               cwd_value->ToString()),
                   true);
@@ -1368,12 +1455,12 @@ void GenericRuleWithDeps(
     for (auto const& dep : dependency_values) {
         inputs = ExpressionPtr{Expression::map_t{inputs, (*dep)->RunFiles()}};
     }
-    auto inputs_conflict = BuildMaps::Target::Utils::tree_conflict(inputs);
     for (auto const& dep : dependency_values) {
         inputs = ExpressionPtr{Expression::map_t{inputs, (*dep)->Artifacts()}};
     }
+    auto inputs_conflict = BuildMaps::Target::Utils::tree_conflict(inputs);
     // While syntactical conflicts are resolved in a latest wins (with artifacts
-    // after runfiles), semantic path conclicts are an error.
+    // after runfiles), semantic path conflicts are an error.
     if (inputs_conflict) {
         (*logger)(fmt::format("Input artifacts have staging conflict on {}",
                               nlohmann::json(*inputs_conflict).dump()),
@@ -1431,6 +1518,7 @@ void GenericRuleWithDeps(
         std::vector<ActionDescription::Ptr>{action},
         std::vector<std::string>{},
         std::move(trees),
+        std::vector<TreeOverlay::Ptr>{},
         std::move(effective_vars),
         std::move(tainted),
         std::move(implied_export),
@@ -1643,6 +1731,7 @@ void ConfigureRule(
                 std::vector<ActionDescription::Ptr>{},
                 std::vector<std::string>{},
                 std::vector<Tree::Ptr>{},
+                std::vector<TreeOverlay::Ptr>{},
                 std::move(vars_set),
                 tainted,
                 configured_target->ImpliedExport(),
@@ -1668,6 +1757,8 @@ auto const kBuiltIns = std::unordered_map<
     {"export", ExportRule},
     {"file_gen", FileGenRule},
     {"tree", TreeRule},
+    {"tree_overlay", TreeOverlayRule},
+    {"disjoint_tree_overlay", DisjointTreeOverlayRule},
     {"symlink", SymlinkRule},
     {"generic", GenericRule},
     {"install", InstallRule},
