@@ -16,7 +16,9 @@
 #define INCLUDED_SRC_BUILDTOOL_BUILD_ENGINE_TARGET_MAP_CONFIGURED_TARGET_HPP
 
 #include <cstddef>
+#include <functional>
 #include <memory>
+#include <string>
 
 #include "fmt/core.h"
 #include "src/buildtool/build_engine/base_maps/entity_name_data.hpp"
@@ -46,6 +48,17 @@ struct ConfiguredTarget {
             target.ToString(),
             AbbreviateJson(PruneJson(config.ToJson()), config_length));
     }
+
+    [[nodiscard]] auto operator<(ConfiguredTarget const& other) const noexcept
+        -> bool {
+        if (target < other.target) {
+            return true;
+        }
+        if (other.target < target) {
+            return false;
+        }
+        return config < other.config;
+    }
 };
 
 using ConfiguredTargetPtr = std::shared_ptr<ConfiguredTarget>;
