@@ -29,7 +29,6 @@
 #include "src/buildtool/logging/logger.hpp"
 #include "src/utils/cpp/expected.hpp"
 #include "src/utils/cpp/hex_string.hpp"
-#include "src/utils/cpp/path.hpp"
 
 namespace {
 [[nodiscard]] auto CreateObjectInfo(HashFunction hash_function,
@@ -104,12 +103,6 @@ auto TreeReaderUtils::ReadObjectInfos(bazel_re::Directory const& dir,
         }
 
         for (auto const& l : dir.symlinks()) {
-            // check validity of symlinks
-            if (not PathIsNonUpwards(l.target())) {
-                Logger::Log(
-                    LogLevel::Error, "found invalid symlink at {}", l.name());
-                return false;
-            }
             if (not store_info(l.name(), CreateObjectInfo(hash_function, l))) {
                 return false;
             }
