@@ -15,6 +15,7 @@
 #include "src/buildtool/execution_api/execution_service/execution_server.hpp"
 
 #include <chrono>
+#include <cstdint>
 #include <filesystem>
 #include <fstream>
 #include <functional>
@@ -36,30 +37,40 @@
 #include "src/buildtool/common/artifact_digest_factory.hpp"
 #include "src/buildtool/common/protocol_traits.hpp"
 #include "src/buildtool/crypto/hash_function.hpp"
+#include "src/buildtool/execution_api/common/execution_response.hpp"
 #include "src/buildtool/execution_api/execution_service/operation_cache.hpp"
 #include "src/buildtool/execution_api/local/local_cas_reader.hpp"
+#include "src/buildtool/execution_api/local/local_response.hpp"
 #include "src/buildtool/file_system/file_system_manager.hpp"
 #include "src/buildtool/file_system/object_type.hpp"
 #include "src/buildtool/logging/log_level.hpp"
 #include "src/buildtool/storage/garbage_collector.hpp"
 #include "src/utils/cpp/hex_string.hpp"
-#include "src/utils/cpp/path.hpp"
 
 namespace {
+void SetTimeStamp(
+    gsl::not_null<::google::protobuf::Timestamp*> const& t,
+    std::chrono::time_point<std::chrono::high_resolution_clock> const& tvalue) {
+    const int64_t k_nanoseconds_per_second = 1'000'000'000;
+    auto nanos = std::chrono::duration_cast<std::chrono::nanoseconds>(
+                     tvalue.time_since_epoch())
+                     .count();
+    t->set_seconds(nanos / k_nanoseconds_per_second);
+    t->set_nanos(static_cast<int32_t>(nanos % k_nanoseconds_per_second));
+}
+
 void UpdateTimeStamp(
     gsl::not_null<::google::longrunning::Operation*> const& op) {
     ::google::protobuf::Timestamp t;
-    t.set_seconds(
-        std::chrono::duration_cast<std::chrono::seconds>(
-            std::chrono::high_resolution_clock::now().time_since_epoch())
-            .count());
+    SetTimeStamp(&t, std::chrono::high_resolution_clock::now());
     op->mutable_metadata()->PackFrom(t);
 }
 
 [[nodiscard]] auto ToBazelActionResult(
-    IExecutionResponse::ArtifactInfos const& artifacts,
-    IExecutionResponse::DirSymlinks const& dir_symlinks,
-    Storage const& storage) noexcept
+    LocalResponse::ArtifactInfos const& artifacts,
+    LocalResponse::DirSymlinks const* dir_symlinks,
+    Storage const& storage,
+    bool legacy_client) noexcept
     -> expected<bazel_re::ActionResult, std::string>;
 
 [[nodiscard]] auto ToBazelAction(ArtifactDigest const& action_digest,
@@ -73,8 +84,8 @@ void UpdateTimeStamp(
 
 auto ExecutionServiceImpl::ToIExecutionAction(
     ::bazel_re::Action const& action,
-    ::bazel_re::Command const& command) const noexcept
-    -> std::optional<IExecutionAction::Ptr> {
+    ::bazel_re::Command const& command,
+    bool legacy_client) const noexcept -> std::optional<IExecutionAction::Ptr> {
     auto const root_digest = ArtifactDigestFactory::FromBazel(
         storage_config_.hash_function.GetType(), action.input_root_digest());
     if (not root_digest) {
@@ -82,21 +93,38 @@ auto ExecutionServiceImpl::ToIExecutionAction(
     }
     std::vector<std::string> const args(command.arguments().begin(),
                                         command.arguments().end());
-    std::vector<std::string> const files(command.output_files().begin(),
-                                         command.output_files().end());
-    std::vector<std::string> const dirs(command.output_directories().begin(),
-                                        command.output_directories().end());
     std::map<std::string, std::string> env_vars;
     for (auto const& x : command.environment_variables()) {
         env_vars.insert_or_assign(x.name(), x.value());
     }
-    auto execution_action = api_.CreateAction(*root_digest,
-                                              args,
-                                              command.working_directory(),
-                                              files,
-                                              dirs,
-                                              env_vars,
-                                              /*properties=*/{});
+    auto execution_action = IExecutionAction::Ptr{};
+    if (legacy_client) {
+        // force legacy mode, DEPRECATED as of RBEv2.1
+        std::vector<std::string> const files(command.output_files().begin(),
+                                             command.output_files().end());
+        std::vector<std::string> const dirs(
+            command.output_directories().begin(),
+            command.output_directories().end());
+        execution_action = api_.CreateAction(*root_digest,
+                                             args,
+                                             command.working_directory(),
+                                             files,
+                                             dirs,
+                                             env_vars,
+                                             /*properties=*/{},
+                                             /*force_legacy=*/true);
+    }
+    else {
+        std::vector<std::string> const paths(command.output_paths().begin(),
+                                             command.output_paths().end());
+        execution_action = api_.CreateAction(*root_digest,
+                                             args,
+                                             command.working_directory(),
+                                             paths,
+                                             env_vars,
+                                             /*properties=*/{});
+    }
+
     if (execution_action == nullptr) {
         return std::nullopt;
     }
@@ -108,47 +136,53 @@ auto ExecutionServiceImpl::ToIExecutionAction(
 }
 
 auto ExecutionServiceImpl::ToBazelExecuteResponse(
-    IExecutionResponse::Ptr const& i_execution_response) const noexcept
+    gsl::not_null<LocalResponse*> const& local_response,
+    bool legacy_client) const noexcept
     -> expected<::bazel_re::ExecuteResponse, std::string> {
-    auto artifacts = i_execution_response->Artifacts();
+    auto artifacts = local_response->Artifacts();
     if (not artifacts) {
         return unexpected{std::move(artifacts).error()};
     }
-    auto dir_symlinks = i_execution_response->DirectorySymlinks();
-    if (not dir_symlinks) {
+    auto dir_symlinks = local_response->DirectorySymlinks();
+    LocalResponse::DirSymlinks const* dir_symlinks_ptr{};
+    if (dir_symlinks) {
+        dir_symlinks_ptr = *dir_symlinks;
+    }
+    else if (legacy_client) {
+        // For legacy clients (<v2.1), it is required to populate output file
+        // and directory symlinks separately.
         return unexpected{std::move(dir_symlinks).error()};
     }
     auto result = ToBazelActionResult(*std::move(artifacts).value(),
-                                      *std::move(dir_symlinks).value(),
-                                      storage_);
+                                      dir_symlinks_ptr,
+                                      storage_,
+                                      legacy_client);
     if (not result) {
         return unexpected{std::move(result).error()};
     }
 
     auto action_result = *std::move(result);
 
-    action_result.set_exit_code(i_execution_response->ExitCode());
-    if (i_execution_response->HasStdErr()) {
+    action_result.set_exit_code(local_response->ExitCode());
+    if (local_response->HasStdErr()) {
         auto const cas_digest =
-            storage_.CAS().StoreBlob(i_execution_response->StdErr(),
+            storage_.CAS().StoreBlob(local_response->StdErr(),
                                      /*is_executable=*/false);
         if (not cas_digest) {
-            return unexpected{
-                fmt::format("Could not store stderr of action {}",
-                            i_execution_response->ActionDigest())};
+            return unexpected{fmt::format("Could not store stderr of action {}",
+                                          local_response->ActionDigest())};
         }
         (*action_result.mutable_stderr_digest()) =
             ArtifactDigestFactory::ToBazel(*cas_digest);
     }
 
-    if (i_execution_response->HasStdOut()) {
+    if (local_response->HasStdOut()) {
         auto const cas_digest =
-            storage_.CAS().StoreBlob(i_execution_response->StdOut(),
+            storage_.CAS().StoreBlob(local_response->StdOut(),
                                      /*is_executable=*/false);
         if (not cas_digest) {
-            return unexpected{
-                fmt::format("Could not store stdout of action {}",
-                            i_execution_response->ActionDigest())};
+            return unexpected{fmt::format("Could not store stdout of action {}",
+                                          local_response->ActionDigest())};
         }
         (*action_result.mutable_stdout_digest()) =
             ArtifactDigestFactory::ToBazel(*cas_digest);
@@ -156,7 +190,7 @@ auto ExecutionServiceImpl::ToBazelExecuteResponse(
 
     ::bazel_re::ExecuteResponse bazel_response{};
     (*bazel_response.mutable_result()) = std::move(action_result);
-    bazel_response.set_cached_result(i_execution_response->IsCached());
+    bazel_response.set_cached_result(local_response->IsCached());
 
     // we run the action locally, so no communication issues should happen
     bazel_response.mutable_status()->set_code(grpc::StatusCode::OK);
@@ -207,7 +241,10 @@ auto ExecutionServiceImpl::Execute(
         return ::grpc::Status{grpc::StatusCode::INTERNAL,
                               std::move(command).error()};
     }
-    auto i_execution_action = ToIExecutionAction(*action, *command);
+    // If output_paths is empty, the client is using legacy API (<v2.1)
+    bool legacy_client = command->output_paths().empty();
+    auto i_execution_action =
+        ToIExecutionAction(*action, *command, legacy_client);
     if (not i_execution_action) {
         auto const str = fmt::format("Could not create action from {}",
                                      action_digest->hash());
@@ -233,24 +270,39 @@ auto ExecutionServiceImpl::Execute(
         action_digest->hash(),
         std::chrono::duration_cast<std::chrono::seconds>(t1 - t0).count());
 
-    if (i_execution_response == nullptr) {
-        auto const str =
-            fmt::format("Failed to execute action {}", action_digest->hash());
-        logger_.Emit(LogLevel::Error, "{}", str);
-        return ::grpc::Status{grpc::StatusCode::INTERNAL, str};
+    auto* local_response =
+        dynamic_cast<LocalResponse*>(i_execution_response.get());
+    if (local_response == nullptr) {
+        auto error_msg =
+            (i_execution_response == nullptr)
+                ? fmt::format("Failed to execute action {}",
+                              action_digest->hash())
+                : std::string{"Local action did not produce a local response"};
+        logger_.Emit(LogLevel::Error, "{}", error_msg);
+        return ::grpc::Status{grpc::StatusCode::INTERNAL, error_msg};
     }
 
-    auto execute_response = ToBazelExecuteResponse(i_execution_response);
+    auto execute_response =
+        ToBazelExecuteResponse(local_response, legacy_client);
     if (not execute_response) {
         logger_.Emit(LogLevel::Error, "{}", execute_response.error());
         return ::grpc::Status{grpc::StatusCode::INTERNAL,
                               std::move(execute_response).error()};
     }
+    ::bazel_re::ExecuteResponse response = *execute_response;
+    SetTimeStamp(response.mutable_result()
+                     ->mutable_execution_metadata()
+                     ->mutable_worker_start_timestamp(),
+                 t0);
+    SetTimeStamp(response.mutable_result()
+                     ->mutable_execution_metadata()
+                     ->mutable_worker_completed_timestamp(),
+                 t1);
 
     // Store the result in action cache
     if (i_execution_response->ExitCode() == 0 and not action->do_not_cache()) {
-        if (not storage_.ActionCache().StoreResult(
-                *action_digest, execute_response->result())) {
+        if (not storage_.ActionCache().StoreResult(*action_digest,
+                                                   response.result())) {
             auto const str =
                 fmt::format("Could not store action result for action {}",
                             action_digest->hash());
@@ -260,7 +312,7 @@ auto ExecutionServiceImpl::Execute(
         }
     }
 
-    WriteResponse(*execute_response, writer, std::move(op));
+    WriteResponse(response, writer, std::move(op));
     return ::grpc::Status::OK;
 }
 
@@ -302,13 +354,7 @@ namespace {
 
     LocalCasReader reader(&storage.CAS());
     if (ProtocolTraits::IsNative(storage.GetHashFunction().GetType())) {
-        // In native mode: Check validity of tree entries, otherwise set the
-        // digest directly.
-        if (not reader.ReadGitTree(digest)) {
-            auto const error = fmt::format(
-                "Found invalid entry in the Git Tree {}", digest.hash());
-            return unexpected{error};
-        }
+        // In native mode: set the digest directly.
         (*out_dir.mutable_tree_digest()) =
             ArtifactDigestFactory::ToBazel(digest);
     }
@@ -355,13 +401,6 @@ namespace {
             "Failed to read the symlink content for {}", digest.hash())};
     }
 
-    // in native mode, check that we do not pass invalid symlinks
-    if (ProtocolTraits::IsNative(storage.GetHashFunction().GetType()) and
-        not PathIsNonUpwards(*content)) {
-        auto const error = fmt::format("Invalid symlink for {}", digest.hash());
-        return unexpected{error};
-    }
-
     *(out_link.mutable_target()) = *std::move(content);
     return out_link;
 }
@@ -377,21 +416,33 @@ namespace {
 }
 
 [[nodiscard]] auto ToBazelActionResult(
-    IExecutionResponse::ArtifactInfos const& artifacts,
-    IExecutionResponse::DirSymlinks const& dir_symlinks,
-    Storage const& storage) noexcept
+    LocalResponse::ArtifactInfos const& artifacts,
+    LocalResponse::DirSymlinks const* dir_symlinks,
+    Storage const& storage,
+    bool legacy_client) noexcept
     -> expected<bazel_re::ActionResult, std::string> {
+    if (legacy_client and dir_symlinks == nullptr) {
+        return unexpected{std::string{
+            "Cannot create action result for clients using RBE protocol <v2.1 "
+            "without output_directory_symlinks reported by the local API."}};
+    }
     bazel_re::ActionResult result{};
     auto& result_files = *result.mutable_output_files();
     auto& result_file_links = *result.mutable_output_file_symlinks();
     auto& result_dirs = *result.mutable_output_directories();
     auto& result_dir_links = *result.mutable_output_directory_symlinks();
+    auto& result_links = *result.mutable_output_symlinks();
 
     auto const size = static_cast<int>(artifacts.size());
     result_files.Reserve(size);
-    result_file_links.Reserve(size);
     result_dirs.Reserve(size);
-    result_dir_links.Reserve(size);
+    if (not legacy_client) {
+        result_links.Reserve(size);
+    }
+    if (dir_symlinks != nullptr) {
+        result_file_links.Reserve(size);
+        result_dir_links.Reserve(size);
+    }
 
     for (auto const& [path, info] : artifacts) {
         if (info.type == ObjectType::Tree) {
@@ -406,13 +457,21 @@ namespace {
             if (not out_link) {
                 return unexpected{std::move(out_link).error()};
             }
-            if (dir_symlinks.contains(path)) {
-                // directory symlink
-                result_dir_links.Add(*std::move(out_link));
+            if (not legacy_client) {
+                // only set generic 'output symlinks' if the client was not
+                // using legacy protocol (<v2.1) without 'output_paths'.
+                *result_links.Add() = *out_link;
             }
-            else {
-                // file symlinks
-                result_file_links.Add(*std::move(out_link));
+            if (dir_symlinks != nullptr) {
+                // DEPRECATED as of v2.1, still needed for v2.0
+                if (dir_symlinks->contains(path)) {
+                    // directory symlink
+                    result_dir_links.Add(*std::move(out_link));
+                }
+                else {
+                    // file symlinks
+                    result_file_links.Add(*std::move(out_link));
+                }
             }
         }
         else {
