@@ -14,6 +14,10 @@
 
 #include "src/buildtool/execution_api/execution_service/operations_server.hpp"
 
+#include <optional>
+#include <string>
+#include <utility>
+
 #include "fmt/core.h"
 #include "src/buildtool/execution_api/execution_service/operation_cache.hpp"
 #include "src/buildtool/logging/log_level.hpp"
