@@ -17,13 +17,14 @@
 
 #include <cstddef>
 #include <optional>
+#include <string>
 
 #include "gsl/gsl"
-#include "src/buildtool/file_system/object_type.hpp"
 
 extern "C" {
 struct git_oid;
 struct git_odb;
+struct git_repository;
 struct git_tree;
 struct git_signature;
 struct git_object;
@@ -47,6 +48,8 @@ constexpr std::size_t kGitLockNumTries{10};
 
 void odb_closer(gsl::owner<git_odb*> odb);
 
+void repository_closer(gsl::owner<git_repository*> repository);
+
 void tree_closer(gsl::owner<git_tree*> tree);
 
 void signature_closer(gsl::owner<git_signature*> signature);
