@@ -17,25 +17,32 @@
 #ifndef BOOTSTRAP_BUILD_TOOL
 
 #include <chrono>
+#include <cmath>
 #include <cstddef>
 #include <cstdlib>
+#include <exception>
 #include <filesystem>
 #include <fstream>
 #include <optional>
+#include <ratio>
 #include <string>
 #include <utility>
-#include <variant>
 #include <vector>
 
 #include "nlohmann/json.hpp"
 #include "src/buildtool/build_engine/expression/configuration.hpp"
 #include "src/buildtool/build_engine/expression/expression.hpp"
+#include "src/buildtool/build_engine/expression/expression_ptr.hpp"
+#include "src/buildtool/common/cli.hpp"
 #include "src/buildtool/common/location.hpp"
+#include "src/buildtool/common/retry_cli.hpp"
 #include "src/buildtool/crypto/hash_function.hpp"
 #include "src/buildtool/file_system/file_system_manager.hpp"
 #include "src/buildtool/logging/log_level.hpp"
 #include "src/buildtool/logging/logger.hpp"
+#include "src/buildtool/main/build_utils.hpp"
 #include "src/buildtool/main/exit_codes.hpp"
+#include "src/utils/cpp/expected.hpp"
 
 namespace {
 
@@ -408,6 +415,22 @@ void ReadJustServeConfig(gsl::not_null<CommandLineArguments*> const& clargs) {
             }
             clargs->endpoint.remote_execution_address = address->String();
         }
+        // read the address used by the client
+        auto client_address =
+            exec_endpoint->Get("client address", Expression::none_t{});
+        if (client_address.IsNotNull()) {
+            if (not client_address->IsString()) {
+                Logger::Log(
+                    LogLevel::Error,
+                    "In serve service config file {}:\nValue for execution "
+                    "endpoint key \"client address\" has to be a string, but "
+                    "found {}",
+                    clargs->serve.config.string(),
+                    client_address->ToString());
+                std::exit(kExitFailure);
+            }
+            clargs->serve.client_remote_address = client_address->String();
+        }
         if (not ParseRetryCliOptions(serve_config, &clargs->retry)) {
             std::exit(kExitFailure);
         }
