@@ -60,8 +60,7 @@ auto RepositoryGarbageCollector::TriggerGarbageCollection(
             return false;
         }
         if (FileSystemManager::IsDirectory(remove_me)) {
-            if (not FileSystemManager::RemoveDirectory(remove_me,
-                                                       /*recursively=*/true)) {
+            if (not FileSystemManager::RemoveDirectory(remove_me)) {
                 Logger::Log(LogLevel::Error,
                             "Failed to remove directory {}",
                             remove_me.string());
@@ -136,8 +135,7 @@ auto RepositoryGarbageCollector::TriggerGarbageCollection(
             return false;
         }
         if (FileSystemManager::IsDirectory(remove_me)) {
-            if (not FileSystemManager::RemoveDirectory(remove_me,
-                                                       /*recursively=*/true)) {
+            if (not FileSystemManager::RemoveDirectory(remove_me)) {
                 Logger::Log(LogLevel::Error,
                             "Failed to remove directory {}",
                             remove_me.string());
