@@ -17,11 +17,14 @@
 #include <algorithm>
 #include <exception>
 #include <iterator>
+#include <memory>
 #include <string>
 #include <vector>
 
+#include "src/buildtool/build_engine/expression/expression_ptr.hpp"
 #include "src/buildtool/common/artifact_digest.hpp"
 #include "src/buildtool/common/artifact_digest_factory.hpp"
+#include "src/buildtool/file_system/object_type.hpp"
 #include "src/buildtool/logging/log_level.hpp"
 #include "src/buildtool/logging/logger.hpp"
 #include "src/utils/cpp/expected.hpp"
