@@ -15,15 +15,23 @@
 #include "src/other_tools/git_operations/git_repo_remote.hpp"
 
 #include <cstddef>
+#include <cstdlib>
+#include <exception>
+#include <functional>
+#include <limits>
+#include <map>
 #include <utility>  // std::move
 
 #include "fmt/core.h"
+#include "gsl/gsl"
 #include "nlohmann/json.hpp"
+#include "src/buildtool/file_system/file_system_manager.hpp"
 #include "src/buildtool/file_system/git_utils.hpp"
 #include "src/buildtool/logging/log_level.hpp"
 #include "src/buildtool/logging/logger.hpp"
 #include "src/buildtool/system/system_command.hpp"
 #include "src/other_tools/git_operations/git_config_settings.hpp"
+#include "src/utils/cpp/tmp_dir.hpp"
 
 extern "C" {
 #include <git2.h>
@@ -57,21 +65,17 @@ struct FetchIntoODBBackend {
                                            [[maybe_unused]] git_odb* odb,
                                            git_indexer_progress_cb progress_cb,
                                            void* progress_payload) -> int {
-    if (_backend != nullptr) {
-        auto* b = reinterpret_cast<FetchIntoODBBackend*>(_backend);  // NOLINT
-        return git_odb_write_pack(
-            _writepack, b->target_odb, progress_cb, progress_payload);
-    }
-    return GIT_ERROR;
+    Ensures(_backend != nullptr);
+    auto* b = reinterpret_cast<FetchIntoODBBackend*>(_backend);  // NOLINT
+    return git_odb_write_pack(
+        _writepack, b->target_odb, progress_cb, progress_payload);
 }
 
 [[nodiscard]] auto fetch_backend_exists(git_odb_backend* _backend,
                                         const git_oid* oid) -> int {
-    if (_backend != nullptr) {
-        auto* b = reinterpret_cast<FetchIntoODBBackend*>(_backend);  // NOLINT
-        return git_odb_exists(b->target_odb, oid);
-    }
-    return GIT_ERROR;
+    Ensures(_backend != nullptr);
+    auto* b = reinterpret_cast<FetchIntoODBBackend*>(_backend);  // NOLINT
+    return git_odb_exists(b->target_odb, oid);
 }
 
 void fetch_backend_free(git_odb_backend* /*_backend*/) {}
@@ -94,7 +98,7 @@ auto const kFetchIntoODBParent = CreateFetchIntoODBParent();
 auto GitRepoRemote::Open(GitCASPtr git_cas) noexcept
     -> std::optional<GitRepoRemote> {
     auto repo = GitRepoRemote(std::move(git_cas));
-    if (not repo.GetRepoRef()) {
+    if (not repo.GetGitCAS()) {
         return std::nullopt;
     }
     return repo;
@@ -103,7 +107,7 @@ auto GitRepoRemote::Open(GitCASPtr git_cas) noexcept
 auto GitRepoRemote::Open(std::filesystem::path const& repo_path) noexcept
     -> std::optional<GitRepoRemote> {
     auto repo = GitRepoRemote(repo_path);
-    if (not repo.GetRepoRef()) {
+    if (not repo.GetGitCAS()) {
         return std::nullopt;
     }
     return repo;
@@ -151,12 +155,13 @@ auto GitRepoRemote::GetCommitFromRemote(std::shared_ptr<git_config> cfg,
         }
         // create remote
         git_remote* remote_ptr{nullptr};
-        if (git_remote_create_anonymous(
-                &remote_ptr, GetRepoRef()->Ptr(), repo_url.c_str()) != 0) {
+        if (git_remote_create_anonymous(&remote_ptr,
+                                        GetGitCAS()->GetRepository(),
+                                        repo_url.c_str()) != 0) {
             (*logger)(
                 fmt::format("Creating anonymous remote for git repository {} "
                             "failed with:\n{}",
-                            GetGitPath().string(),
+                            GetGitCAS()->GetPath().string(),
                             GitLastError()),
                 true /*fatal*/);
             git_remote_free(remote_ptr);
@@ -222,7 +227,7 @@ auto GitRepoRemote::GetCommitFromRemote(std::shared_ptr<git_config> cfg,
                 fmt::format("Connecting to remote {} for git repository {} "
                             "failed with:\n{}",
                             repo_url,
-                            GetGitPath().string(),
+                            GetGitCAS()->GetPath().string(),
                             GitLastError()),
                 true /*fatal*/);
             return std::nullopt;
@@ -282,12 +287,13 @@ auto GitRepoRemote::FetchFromRemote(std::shared_ptr<git_config> cfg,
         }
         // create remote from repo
         git_remote* remote_ptr{nullptr};
-        if (git_remote_create_anonymous(
-                &remote_ptr, GetRepoRef()->Ptr(), repo_url.c_str()) != 0) {
+        if (git_remote_create_anonymous(&remote_ptr,
+                                        GetGitCAS()->GetRepository(),
+                                        repo_url.c_str()) != 0) {
             (*logger)(fmt::format("Creating remote {} for git repository {} "
                                   "failed with:\n{}",
                                   repo_url,
-                                  GetGitPath().string(),
+                                  GetGitCAS()->GetPath().string(),
                                   GitLastError()),
                       true /*fatal*/);
             // cleanup resources
@@ -304,8 +310,8 @@ auto GitRepoRemote::FetchFromRemote(std::shared_ptr<git_config> cfg,
         if (not cfg) {
             // get config snapshot of current repo
             git_config* cfg_ptr{nullptr};
-            if (git_repository_config_snapshot(&cfg_ptr, GetRepoRef()->Ptr()) !=
-                0) {
+            if (git_repository_config_snapshot(
+                    &cfg_ptr, GetGitCAS()->GetRepository()) != 0) {
                 (*logger)(fmt::format("Retrieving config object in fetch from "
                                       "remote failed with:\n{}",
                                       GitLastError()),
@@ -375,7 +381,7 @@ auto GitRepoRemote::FetchFromRemote(std::shared_ptr<git_config> cfg,
                 fmt::format("Fetching{} in git repository {} failed "
                             "with:\n{}",
                             branch ? fmt::format(" branch {}", *branch) : "",
-                            GetGitPath().string(),
+                            GetGitCAS()->GetPath().string(),
                             GitLastError()),
                 true /*fatal*/);
             return false;
@@ -459,11 +465,11 @@ auto GitRepoRemote::UpdateCommitViaTmpRepo(
         }
         // set up the system command
         SystemCommand system{repo_url};
-        auto const exit_code =
-            system.Execute(cmdline,
-                           env,
-                           GetGitPath(),  // which path is not actually relevant
-                           tmp_path);
+        auto const exit_code = system.Execute(
+            cmdline,
+            env,
+            GetGitCAS()->GetPath(),  // which path is not actually relevant
+            tmp_path);
 
         if (not exit_code) {
             (*logger)(fmt::format("exec() on command failed."),
@@ -562,9 +568,9 @@ auto GitRepoRemote::FetchViaTmpRepo(StorageConfig const& storage_config,
             }
             // add backend, with max priority
             FetchIntoODBBackend b{.parent = kFetchIntoODBParent,
-                                  .target_odb = GetGitOdb().get()};
+                                  .target_odb = GetGitCAS()->GetODB()};
             if (git_odb_add_backend(
-                    tmp_repo->GetGitOdb().get(),
+                    tmp_repo->GetGitCAS()->GetODB(),
                     // NOLINTNEXTLINE(cppcoreguidelines-pro-type-reinterpret-cast)
                     reinterpret_cast<git_odb_backend*>(&b),
                     std::numeric_limits<int>::max()) == 0) {
@@ -620,7 +626,7 @@ auto GitRepoRemote::FetchViaTmpRepo(StorageConfig const& storage_config,
         // run command
         SystemCommand system{repo_url};
         auto const exit_code =
-            system.Execute(cmdline, env, GetGitPath(), tmp_path);
+            system.Execute(cmdline, env, GetGitCAS()->GetPath(), tmp_path);
 
         if (not exit_code) {
             (*logger)(fmt::format("exec() on command failed."),
