@@ -86,8 +86,7 @@ auto MultiRepoFetch(std::shared_ptr<Configuration> const& config,
     if (not fetch_dir) {
         for (auto const& d : common_args.just_mr_paths->distdirs) {
             if (FileSystemManager::IsDirectory(d)) {
-                fetch_dir = std::filesystem::weakly_canonical(
-                    std::filesystem::absolute(d));
+                fetch_dir = std::filesystem::weakly_canonical(d);
                 break;
             }
         }
@@ -147,9 +146,8 @@ auto MultiRepoFetch(std::shared_ptr<Configuration> const& config,
                     std::filesystem::path(repo_path->String());
                 if (not repo_path_as_path.is_absolute()) {
                     repo_path_as_path = std::filesystem::weakly_canonical(
-                        std::filesystem::absolute(
-                            common_args.just_mr_paths->setup_root /
-                            repo_path_as_path));
+                        common_args.just_mr_paths->setup_root /
+                        repo_path_as_path);
                 }
                 // only warn if repo workspace differs to invocation workspace
                 if (not is_subpath(
@@ -225,7 +223,8 @@ auto MultiRepoFetch(std::shared_ptr<Configuration> const& config,
                 return kExitFetchError;
             }
             auto repo_type_str = repo_type->get()->String();
-            if (not kCheckoutTypeMap.contains(repo_type_str)) {
+            auto const checkout_type_it = kCheckoutTypeMap.find(repo_type_str);
+            if (checkout_type_it == kCheckoutTypeMap.end()) {
                 Logger::Log(LogLevel::Error,
                             "Config: Unknown repository type {} for {}",
                             nlohmann::json(repo_type_str).dump(),
@@ -233,7 +232,7 @@ auto MultiRepoFetch(std::shared_ptr<Configuration> const& config,
                 return kExitFetchError;
             }
             // only do work if repo is archive or git tree type
-            switch (kCheckoutTypeMap.at(repo_type_str)) {
+            switch (checkout_type_it->second) {
                 case CheckoutType::Archive: {
                     auto logger = std::make_shared<AsyncMapConsumerLogger>(
                         [&repo_name](std::string const& msg, bool fatal) {
@@ -426,7 +425,8 @@ auto MultiRepoFetch(std::shared_ptr<Configuration> const& config,
                                                 &*auth_config,
                                                 &*retry_config,
                                                 config,
-                                                hash_fct);
+                                                hash_fct,
+                                                mr_local_api->GetTempSpace());
     }
     bool const has_remote_api = remote_api != nullptr;
 
