@@ -14,21 +14,30 @@
 
 #include "src/other_tools/root_maps/commit_git_map.hpp"
 
-#include <algorithm>
+#include <filesystem>
+#include <memory>
 #include <optional>
 #include <string>
 
 #include "fmt/core.h"
+#include "src/buildtool/common/artifact.hpp"
+#include "src/buildtool/common/artifact_digest.hpp"
 #include "src/buildtool/common/artifact_digest_factory.hpp"
 #include "src/buildtool/crypto/hash_function.hpp"
 #include "src/buildtool/file_system/file_root.hpp"
 #include "src/buildtool/file_system/file_system_manager.hpp"
+#include "src/buildtool/file_system/git_cas.hpp"
+#include "src/buildtool/file_system/git_repo.hpp"
+#include "src/buildtool/file_system/git_types.hpp"
+#include "src/buildtool/file_system/object_type.hpp"
 #include "src/buildtool/multithreading/task_system.hpp"
+#include "src/buildtool/progress_reporting/task_tracker.hpp"
 #include "src/buildtool/storage/fs_utils.hpp"
+#include "src/other_tools/git_operations/git_ops_types.hpp"
 #include "src/other_tools/git_operations/git_repo_remote.hpp"
-#include "src/other_tools/root_maps/root_utils.hpp"
-#include "src/other_tools/utils/curl_url_handle.hpp"
+#include "src/utils/cpp/expected.hpp"
 #include "src/utils/cpp/path.hpp"
+#include "src/utils/cpp/tmp_dir.hpp"
 
 namespace {
 
@@ -60,23 +69,21 @@ namespace {
 /// root if it was marked absent.
 /// It guarantees the logger is called exactly once with fatal on failure, and
 /// the setter on success.
-void EnsureRootAsAbsent(
-    std::string const& tree_id,
-    std::filesystem::path const& repo_root,
-    GitRepoInfo const& repo_info,
-    ServeApi const* serve,
-    gsl::not_null<StorageConfig const*> const& native_storage_config,
-    StorageConfig const* compat_storage_config,
-    Storage const* compat_storage,
-    gsl::not_null<IExecutionApi const*> const& local_api,
-    IExecutionApi const* remote_api,
-    CommitGitMap::SetterPtr const& ws_setter,
-    CommitGitMap::LoggerPtr const& logger) {
+void EnsureRootAsAbsent(std::string const& tree_id,
+                        std::filesystem::path const& repo_root,
+                        GitRepoInfo const& repo_info,
+                        ServeApi const* serve,
+                        CommitGitMap::SetterPtr const& ws_setter,
+                        CommitGitMap::LoggerPtr const& logger) {
     // this is an absent root
     if (serve != nullptr) {
         // check if the serve endpoint has this root
-        auto has_tree = CheckServeHasAbsentRoot(*serve, tree_id, logger);
+        auto const has_tree = serve->CheckRootTree(tree_id);
         if (not has_tree) {
+            (*logger)(fmt::format("Checking that the serve endpoint knows tree "
+                                  "{} failed.",
+                                  tree_id),
+                      /*fatal=*/true);
             return;
         }
         if (not *has_tree) {
@@ -108,27 +115,23 @@ void EnsureRootAsAbsent(
                               /*fatal=*/true);
                     return;
                 }
-                if (remote_api == nullptr) {
-                    (*logger)(
-                        fmt::format("Missing or incompatible remote-execution "
-                                    "endpoint needed to sync workspace root {} "
-                                    "with the serve endpoint.",
-                                    tree_id),
-                        /*fatal=*/true);
+
+                auto digest =
+                    ArtifactDigestFactory::Create(HashFunction::Type::GitSHA1,
+                                                  tree_id,
+                                                  /*size_unknown=*/0,
+                                                  /*is_tree=*/true);
+                if (not digest.has_value()) {
+                    (*logger)(std::move(digest).error(), /*fatal=*/true);
                     return;
                 }
+
                 // the tree is known locally, so we can upload it to remote CAS
                 // for the serve endpoint to retrieve it and set up the root
-                if (not EnsureAbsentRootOnServe(*serve,
-                                                tree_id,
-                                                repo_root,
-                                                native_storage_config,
-                                                compat_storage_config,
-                                                compat_storage,
-                                                &*local_api,
-                                                remote_api,
-                                                logger,
-                                                true /*no_sync_is_fatal*/)) {
+                auto uploaded = serve->UploadTree(*digest, repo_root);
+                if (not uploaded.has_value()) {
+                    (*logger)(std::move(uploaded).error().Message(),
+                              /*fatal=*/true);
                     return;
                 }
             }
@@ -410,6 +413,9 @@ void NetworkFetchAndSetPresentRoot(
     all_mirrors.insert(
         all_mirrors.begin(), local_mirrors.begin(), local_mirrors.end());
 
+    auto inherit_env =
+        MirrorsUtils::GetInheritEnv(additional_mirrors, repo_info.inherit_env);
+
     for (auto mirror : all_mirrors) {
         auto mirror_path = GitURLIsPath(mirror);
         if (mirror_path) {
@@ -423,7 +429,7 @@ void NetworkFetchAndSetPresentRoot(
         if (git_repo->FetchViaTmpRepo(native_storage_config,
                                       mirror,
                                       repo_info.branch,
-                                      repo_info.inherit_env,
+                                      inherit_env,
                                       git_bin,
                                       launcher,
                                       wrapped_logger)) {
@@ -527,8 +533,6 @@ void EnsureCommit(
     std::vector<std::string> const& launcher,
     ServeApi const* serve,
     gsl::not_null<StorageConfig const*> const& native_storage_config,
-    StorageConfig const* compat_storage_config,
-    Storage const* compat_storage,
     gsl::not_null<IExecutionApi const*> const& local_api,
     IExecutionApi const* remote_api,
     bool fetch_absent,
@@ -610,11 +614,6 @@ void EnsureCommit(
                     native_storage_config->GitRoot(), /*repo_root*/
                     repo_info,
                     serve,
-                    native_storage_config,
-                    compat_storage_config,
-                    compat_storage,
-                    local_api,
-                    remote_api,
                     ws_setter,
                     logger);
             }
@@ -975,14 +974,8 @@ void EnsureCommit(
 
                                 return;
                             }
-                            // just serve should have made the tree available in
-                            // the remote CAS, so log this attempt and revert to
-                            // network
-                            (*logger)(fmt::format("Tree {} marked as served, "
-                                                  "but not found on remote",
-                                                  root_tree_id),
-                                      /*fatal=*/false);
-
+                            // just serve did not make the tree available in
+                            // the remote CAS, so fall back to network
                             NetworkFetchAndSetPresentRoot(
                                 repo_info,
                                 repo_root,
@@ -1061,17 +1054,8 @@ void EnsureCommit(
         // set the workspace root
         if (repo_info.absent and not fetch_absent) {
             // try by all available means to generate and set the absent root
-            EnsureRootAsAbsent(subtree,
-                               repo_root,
-                               repo_info,
-                               serve,
-                               native_storage_config,
-                               compat_storage_config,
-                               compat_storage,
-                               local_api,
-                               remote_api,
-                               ws_setter,
-                               logger);
+            EnsureRootAsAbsent(
+                subtree, repo_root, repo_info, serve, ws_setter, logger);
         }
         else {
             // set root as present
@@ -1099,8 +1083,6 @@ auto CreateCommitGitMap(
     std::vector<std::string> const& launcher,
     ServeApi const* serve,
     gsl::not_null<StorageConfig const*> const& native_storage_config,
-    StorageConfig const* compat_storage_config,
-    Storage const* compat_storage,
     gsl::not_null<IExecutionApi const*> const& local_api,
     IExecutionApi const* remote_api,
     bool fetch_absent,
@@ -1114,8 +1096,6 @@ auto CreateCommitGitMap(
                           launcher,
                           serve,
                           native_storage_config,
-                          compat_storage_config,
-                          compat_storage,
                           local_api,
                           remote_api,
                           fetch_absent,
@@ -1159,8 +1139,6 @@ auto CreateCommitGitMap(
              launcher,
              serve,
              native_storage_config,
-             compat_storage_config,
-             compat_storage,
              local_api,
              remote_api,
              fetch_absent,
@@ -1196,8 +1174,6 @@ auto CreateCommitGitMap(
                              launcher,
                              serve,
                              native_storage_config,
-                             compat_storage_config,
-                             compat_storage,
                              local_api,
                              remote_api,
                              fetch_absent,
