@@ -14,6 +14,10 @@
 
 #include "src/utils/archive/archive_ops.hpp"
 
+#include <cstddef>
+#include <exception>
+#include <memory>
+
 #include "src/buildtool/file_system/file_system_manager.hpp"
 #include "src/buildtool/logging/log_level.hpp"
 #include "src/buildtool/logging/logger.hpp"
