@@ -15,9 +15,12 @@
 #ifndef INCLUDED_SRC_UTILS_AUTOMATA_DFA_MINIMIZER_HPP
 #define INCLUDED_SRC_UTILS_AUTOMATA_DFA_MINIMIZER_HPP
 
+#include <algorithm>
+#include <compare>
 #include <cstddef>
+#include <functional>
+#include <iterator>
 #include <map>
-#include <regex>
 #include <string>
 #include <unordered_map>
 #include <utility>
@@ -45,7 +48,7 @@ class DFAMinimizer {
     // Bucket of states with equal local properties (content and acceptance)
     struct Bucket {
         std::vector<std::string> symbols;
-        states_t states;
+        states_t states{};
     };
 
     // Key used for state pairs. Reordering names will result in the same key.
