@@ -57,9 +57,12 @@
 #include "src/buildtool/file_system/object_type.hpp"
 #include "src/buildtool/logging/logger.hpp"
 #include "src/buildtool/progress_reporting/progress.hpp"
+#include "src/buildtool/storage/config.hpp"
 #include "src/utils/cpp/expected.hpp"
+#include "src/utils/cpp/tmp_dir.hpp"
 #include "test/utils/executor/test_api_bundle.hpp"
 #include "test/utils/hermeticity/test_hash_function_type.hpp"
+#include "test/utils/hermeticity/test_storage_config.hpp"
 
 /// \brief Mockup API test config.
 struct TestApiConfig {
@@ -104,10 +107,21 @@ class TestResponse : public IExecutionResponse {
     [[nodiscard]] auto IsCached() const noexcept -> bool final {
         return config_.response.cached;
     }
+    [[nodiscard]] auto ExecutionDuration() noexcept -> double final {
+        return 0;
+    }
     [[nodiscard]] auto HasStdErr() const noexcept -> bool final { return true; }
     [[nodiscard]] auto HasStdOut() const noexcept -> bool final { return true; }
     [[nodiscard]] auto StdErr() noexcept -> std::string final { return {}; }
     [[nodiscard]] auto StdOut() noexcept -> std::string final { return {}; }
+    [[nodiscard]] auto StdErrDigest() noexcept
+        -> std::optional<ArtifactDigest> final {
+        return std::nullopt;
+    }
+    [[nodiscard]] auto StdOutDigest() noexcept
+        -> std::optional<ArtifactDigest> final {
+        return std::nullopt;
+    }
     [[nodiscard]] auto ActionDigest() const noexcept
         -> std::string const& final {
         static const std::string kEmptyHash;
@@ -120,10 +134,9 @@ class TestResponse : public IExecutionResponse {
         }
         return gsl::not_null<ArtifactInfos const*>(&artifacts_);
     }
-    [[nodiscard]] auto DirectorySymlinks() noexcept
-        -> expected<gsl::not_null<DirSymlinks const*>, std::string> final {
-        static const DirSymlinks kEmptySymlinks{};
-        return gsl::not_null<DirSymlinks const*>(&kEmptySymlinks);
+    [[nodiscard]] auto HasUpwardsSymlinks() noexcept
+        -> expected<bool, std::string> final {
+        return false;
     }
 
   private:
@@ -179,8 +192,11 @@ class TestAction : public IExecutionAction {
 class TestApi : public IExecutionApi {
   public:
     explicit TestApi(TestApiConfig config,
-                     HashFunction::Type hash_type) noexcept
-        : config_{std::move(config)}, hash_type_{hash_type} {}
+                     HashFunction::Type hash_type,
+                     TmpDir::Ptr temp_space) noexcept
+        : config_{std::move(config)},
+          hash_type_{hash_type},
+          temp_space_{std::move(temp_space)} {}
 
     [[nodiscard]] auto CreateAction(
         ArtifactDigest const& /*unused*/,
@@ -189,8 +205,8 @@ class TestApi : public IExecutionApi {
         std::vector<std::string> const& /*unused*/,
         std::vector<std::string> const& /*unused*/,
         std::map<std::string, std::string> const& /*unused*/,
-        std::map<std::string, std::string> const& /*unused*/) const noexcept
-        -> IExecutionAction::Ptr final {
+        std::map<std::string, std::string> const& /*unused*/,
+        bool /*unused*/) const noexcept -> IExecutionAction::Ptr final {
         return std::make_unique<TestAction>(config_);
     }
     [[nodiscard]] auto RetrieveToPaths(
@@ -273,9 +289,14 @@ class TestApi : public IExecutionApi {
         return hash_type_;
     }
 
+    [[nodiscard]] auto GetTempSpace() const noexcept -> TmpDir::Ptr final {
+        return temp_space_;
+    }
+
   private:
     TestApiConfig config_{};
     HashFunction::Type hash_type_;
+    TmpDir::Ptr temp_space_;
 };
 
 [[nodiscard]] auto SetupConfig(std::filesystem::path const& ws)
@@ -323,12 +344,13 @@ class TestApi : public IExecutionApi {
 }
 
 TEST_CASE("Executor: Process artifact", "[executor]") {
+    auto const storage_config = TestStorageConfig::Create();
     std::filesystem::path workspace_path{
         "test/buildtool/execution_engine/executor"};
     DependencyGraph g;
     auto [config, repo_config] = CreateTest(&g, workspace_path);
 
-    HashFunction const hash_function{TestHashType::ReadFromEnvironment()};
+    HashFunction const hash_function = storage_config.Get().hash_function;
 
     auto const local_cpp_id =
         ArtifactDescription::CreateLocal("local.cpp", "").Id();
@@ -345,7 +367,10 @@ TEST_CASE("Executor: Process artifact", "[executor]") {
                                        .exec_config = &remote_config};
 
     SECTION("Processing succeeds for valid config") {
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -353,7 +378,8 @@ TEST_CASE("Executor: Process artifact", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
@@ -363,7 +389,10 @@ TEST_CASE("Executor: Process artifact", "[executor]") {
     SECTION("Processing fails if uploading local artifact failed") {
         config.artifacts[NamedDigest("local.cpp").hash()].uploads = false;
 
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -371,7 +400,8 @@ TEST_CASE("Executor: Process artifact", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(not runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
@@ -381,7 +411,10 @@ TEST_CASE("Executor: Process artifact", "[executor]") {
     SECTION("Processing fails if known artifact is not available") {
         config.artifacts[NamedDigest("known.cpp").hash()].available = false;
 
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -389,7 +422,8 @@ TEST_CASE("Executor: Process artifact", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
@@ -398,13 +432,14 @@ TEST_CASE("Executor: Process artifact", "[executor]") {
 }
 
 TEST_CASE("Executor: Process action", "[executor]") {
+    auto const storage_config = TestStorageConfig::Create();
     std::filesystem::path workspace_path{
         "test/buildtool/execution_engine/executor"};
 
     DependencyGraph g;
     auto [config, repo_config] = CreateTest(&g, workspace_path);
 
-    HashFunction const hash_function{TestHashType::ReadFromEnvironment()};
+    HashFunction const hash_function = storage_config.Get().hash_function;
 
     auto const local_cpp_id =
         ArtifactDescription::CreateLocal("local.cpp", "").Id();
@@ -428,7 +463,10 @@ TEST_CASE("Executor: Process action", "[executor]") {
                                        .exec_config = &remote_config};
 
     SECTION("Processing succeeds for valid config") {
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -436,7 +474,8 @@ TEST_CASE("Executor: Process action", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
@@ -449,7 +488,10 @@ TEST_CASE("Executor: Process action", "[executor]") {
     SECTION("Processing succeeds even if result was is not cached") {
         config.response.cached = false;
 
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -457,7 +499,8 @@ TEST_CASE("Executor: Process action", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
@@ -470,7 +513,10 @@ TEST_CASE("Executor: Process action", "[executor]") {
     SECTION("Processing succeeds even if output is not available in CAS") {
         config.artifacts[NamedDigest("output2.exe").hash()].available = false;
 
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -478,7 +524,8 @@ TEST_CASE("Executor: Process action", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
@@ -494,7 +541,10 @@ TEST_CASE("Executor: Process action", "[executor]") {
     SECTION("Processing fails if execution failed") {
         config.execution.failed = true;
 
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -502,7 +552,8 @@ TEST_CASE("Executor: Process action", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
@@ -515,7 +566,10 @@ TEST_CASE("Executor: Process action", "[executor]") {
     SECTION("Processing fails if exit code is non-zero") {
         config.response.exit_code = 1;
 
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -523,7 +577,8 @@ TEST_CASE("Executor: Process action", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
@@ -539,7 +594,10 @@ TEST_CASE("Executor: Process action", "[executor]") {
     SECTION("Processing fails if any output is missing") {
         config.execution.outputs = {"output1.exe" /*, "output2.exe"*/};
 
-        auto api = std::make_shared<TestApi>(config, hash_function.GetType());
+        auto api = std::make_shared<TestApi>(
+            config,
+            hash_function.GetType(),
+            storage_config.Get().CreateTypedTmpDir("temp_space"));
         Statistics stats{};
         Progress progress{};
         auto const apis = CreateTestApiBundle(api);
@@ -547,7 +605,8 @@ TEST_CASE("Executor: Process action", "[executor]") {
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = &stats,
-                                            .progress = &progress};
+                                            .progress = &progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
 
         CHECK(runner.Process(g.ArtifactNodeWithId(local_cpp_id)));
