@@ -159,7 +159,8 @@ static inline void RunHelloWorldCompilation(
                                         .apis = &apis,
                                         .remote_context = &remote_context,
                                         .statistics = stats,
-                                        .progress = progress};
+                                        .progress = progress,
+                                        .profile = std::nullopt};
     Executor runner{&exec_context};
 
     // upload local artifacts
@@ -290,7 +291,8 @@ static inline void RunGreeterCompilation(
                                         .apis = &apis,
                                         .remote_context = &remote_context,
                                         .statistics = stats,
-                                        .progress = progress};
+                                        .progress = progress,
+                                        .profile = std::nullopt};
     Executor runner{&exec_context};
 
     // upload local artifacts
@@ -457,7 +459,8 @@ static inline void TestUploadAndDownloadTrees(
                                         .apis = &apis,
                                         .remote_context = &remote_context,
                                         .statistics = stats,
-                                        .progress = progress};
+                                        .progress = progress,
+                                        .profile = std::nullopt};
     Executor runner{&exec_context};
 
     REQUIRE(runner.Process(g.ArtifactNodeWithId(foo_id)));
@@ -476,7 +479,7 @@ static inline void TestUploadAndDownloadTrees(
         CHECK(FileSystemManager::IsNonUpwardsSymlink(tmpdir / "b"));
         CHECK(*FileSystemManager::ReadFile(tmpdir / "a") == "foo");
         CHECK(*FileSystemManager::ReadSymlink(tmpdir / "b") == "bar");
-        REQUIRE(FileSystemManager::RemoveDirectory(tmpdir, true));
+        REQUIRE(FileSystemManager::RemoveDirectory(tmpdir));
     }
 
     SECTION("Subdir in tree path") {
@@ -493,7 +496,7 @@ static inline void TestUploadAndDownloadTrees(
         CHECK(FileSystemManager::IsNonUpwardsSymlink(tmpdir / "b" / "a"));
         CHECK(*FileSystemManager::ReadFile(tmpdir / "a") == "foo");
         CHECK(*FileSystemManager::ReadSymlink(tmpdir / "b" / "a") == "bar");
-        REQUIRE(FileSystemManager::RemoveDirectory(tmpdir, true));
+        REQUIRE(FileSystemManager::RemoveDirectory(tmpdir));
     }
 
     SECTION("Nested trees") {
@@ -514,7 +517,7 @@ static inline void TestUploadAndDownloadTrees(
         CHECK(FileSystemManager::IsNonUpwardsSymlink(tmpdir / "b" / "a"));
         CHECK(*FileSystemManager::ReadFile(tmpdir / "a") == "foo");
         CHECK(*FileSystemManager::ReadSymlink(tmpdir / "b" / "a") == "bar");
-        REQUIRE(FileSystemManager::RemoveDirectory(tmpdir, true));
+        REQUIRE(FileSystemManager::RemoveDirectory(tmpdir));
     }
 
     SECTION("Dot-path tree as action input") {
@@ -547,13 +550,16 @@ static inline void TestUploadAndDownloadTrees(
         CHECK(FileSystemManager::IsNonUpwardsSymlink(tmpdir / "b" / "a"));
         CHECK(*FileSystemManager::ReadFile(tmpdir / "a") == "foo");
         CHECK(*FileSystemManager::ReadSymlink(tmpdir / "b" / "a") == "bar");
-        REQUIRE(FileSystemManager::RemoveDirectory(tmpdir, true));
+        REQUIRE(FileSystemManager::RemoveDirectory(tmpdir));
     }
 
     SECTION("Dot-path non-tree as action input") {
         auto action_inputs = ActionDescription::inputs_t{{".", foo_desc}};
         ActionDescription action_desc{
-            {"foo"}, {}, Action{"action_id", {"echo"}, {}}, action_inputs};
+            {"foo"},
+            {},
+            Action{"action_id", std::vector<std::string>{"echo"}, {}},
+            action_inputs};
 
         REQUIRE(g.Add({action_desc}));
         auto const* action_node = g.ActionNodeWithId("action_id");
@@ -629,7 +635,8 @@ static inline void TestRetrieveOutputDirectories(
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = stats,
-                                            .progress = progress};
+                                            .progress = progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
         REQUIRE(runner.Process(action));
 
@@ -683,7 +690,8 @@ static inline void TestRetrieveOutputDirectories(
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = stats,
-                                            .progress = progress};
+                                            .progress = progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
         REQUIRE(runner.Process(action));
 
@@ -754,7 +762,8 @@ static inline void TestRetrieveOutputDirectories(
                                             .apis = &apis,
                                             .remote_context = &remote_context,
                                             .statistics = stats,
-                                            .progress = progress};
+                                            .progress = progress,
+                                            .profile = std::nullopt};
         Executor runner{&exec_context};
         REQUIRE(runner.Process(action));
 
@@ -828,7 +837,8 @@ static inline void TestRetrieveOutputDirectories(
                 .apis = &apis,
                 .remote_context = &remote_context,
                 .statistics = stats,
-                .progress = progress};
+                .progress = progress,
+                .profile = std::nullopt};
             Executor runner{&exec_context};
             CHECK_FALSE(runner.Process(action));
         }
@@ -855,7 +865,8 @@ static inline void TestRetrieveOutputDirectories(
                 .apis = &apis,
                 .remote_context = &remote_context,
                 .statistics = stats,
-                .progress = progress};
+                .progress = progress,
+                .profile = std::nullopt};
             Executor runner{&exec_context};
             CHECK_FALSE(runner.Process(action));
         }
