@@ -16,13 +16,11 @@
 
 #include <filesystem>
 #include <fstream>
-#include <iostream>
 #include <string>
 #include <thread>
 #include <vector>
 
 #include "catch2/catch_test_macros.hpp"
-#include "catch2/generators/catch_generators_all.hpp"
 #include "catch2/matchers/catch_matchers_all.hpp"
 #include "src/buildtool/file_system/file_system_manager.hpp"
 #include "src/buildtool/logging/log_config.hpp"
