@@ -14,7 +14,7 @@
 
 #include "src/utils/cpp/prefix.hpp"
 
-#include <filesystem>
+#include <string>
 
 #include "catch2/catch_test_macros.hpp"
 
